#!/usr/bin/perl -w

# This script may be used to archive w2 products from set time 
# period to a target directory from the source directory given
# below ($dir).

use strict;

my $lev1;

my $numArgs = $#ARGV + 1;
if ($numArgs != 7 && $numArgs != 8) {
  print "Usage:  archive_products.pl <start date> <start hour> <end date> <end hour> <source directory> <target directory> <sources (\"list in quotes\" or \"all\"> <optional list of products in quotes>\n";
  print " e.g.:  archive_radar.pl 20021110 18 20021111 04 /data/realtime/radar /data/demo/20021110 \"KLSX KILX KEAX multi\" \"MergedReflectivityQC MergedAzShear_0-3kmMSL MergedReflectivityQCComposite\"\n";
  print "\n";
  exit;
}

my $start_date = $ARGV[0];
my $start_time = $ARGV[1];
my $end_date = $ARGV[2];
my $end_time = $ARGV[3];
my $dir = $ARGV[4];
my $targetdir = $ARGV[5];
my $sourcelist = $ARGV[6];
my $productlist = "";
if ($numArgs == 8) { $productlist = $ARGV[7];}

if (! -e $targetdir) {
  `mkdir -p $targetdir`;
}

my @sources = split / /,$sourcelist;
my $foo;
foreach $foo (@sources)
{
  print $foo,"\n";
}
chdir $dir;
foreach $lev1 (`/bin/ls -1`) {
  chop $lev1;
  chdir ($dir);
  my $processSource = 0;
  if ($sources[0] eq "all") {
    $processSource = 1;
  } else {
    my $thissource;
    foreach $thissource (@sources) {
      if ($lev1 eq $thissource) {
        $processSource = 1;
      }
    }
  }
  if ($processSource == 1) {
    if (-d $lev1) {
      my $dirtoparse = $dir . "/" . $lev1;
      print "Parsing top-level directory: $dirtoparse\n";
      my $targetdirtoparse = $targetdir . "/" . $lev1;
      parsedir ($dirtoparse,$targetdirtoparse, $start_date, $start_time,
              $end_date, $end_time, $productlist);
      `makeIndex.pl $targetdirtoparse code_index.xml`;
      if (-e "$targetdirtoparse/code_index.lb" ) {
        `/bin/rm $targetdirtoparse/code_index.lb`;
      }
      #`w2simulator -i xml:$targetdirtoparse/code_index.xml -f -l code_index.lb`;
    }
  } 
}


sub parsedir {
  my ($thisdir, $thistargetdir, $startdate, $starttime, $enddate, $endtime, $productlist) = @_;
  my $lev;
  my @products = split / /,$productlist;

  print "Now parsing $thisdir\n";
  chdir ($thisdir);
  foreach $lev (`/bin/ls -1`) {
    chop $lev;
    my $fpath = $thisdir . "/" . $lev;
    if (-d $fpath) {
      #print "$fpath is a directory\n";
      my $newdir = $thisdir . "/" . $lev;
      my $newtargetdir = $thistargetdir . "/" . $lev;
      #print "Now parsing $newdir:\n";
      if (-e $newdir) {
        parsedir($newdir, $newtargetdir, $startdate, $starttime,
              $enddate, $endtime, $productlist);
      } else {
        print "argh -- $newdir doesn't seem to exist\n";
      }
    } else {
      #print "$fpath is a file\n";
      # check to see if we are archiving this product
      my $archiveThisProduct = 0;
      for (my $i=0; $i <= $#products; ++$i) {
        # this only executes if we specified products on the command line,
        # otherwise $#products = -1
        if ($fpath =~ /\/$products[$i]\//) {
          $archiveThisProduct = 1;
        }
      }
      if ($#products == -1) { $archiveThisProduct = 1; }
      if (!$archiveThisProduct) { next; }

      $_ = $lev;
      my ($year1, $mon1, $day1, $hour1, $min1, $sec1);
      my $flag = 0;
      if (/[1-2][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]_at_/) {
        $year1 = substr($lev,0,4);
        $mon1 = substr($lev,5,2);
        $day1 = substr($lev,8,2);
        $hour1 = substr($lev,14,2);
        $min1 = substr($lev,17,2);
        $sec1 = substr($lev,20,2);
        $flag = 1;
      }
      if (/[1-2][0-9][0-9][0-9][0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9][0-9][0-9]./) {
        $year1 = substr($lev,0,4);
        $mon1 = substr($lev,4,2);
        $day1 = substr($lev,6,2);
        $hour1 = substr($lev,9,2);
        $min1 = substr($lev,11,2);
        $sec1 = substr($lev,13,2);
        $flag = 1;
      }
      #if ($lev eq "code_index.lb") {
      #  `/bin/cp $fpath $thistargetdir`;
      #}
      if (!$flag) { 
        print "ERROR: couldn't get a time stamp for $fpath!\n";
      } else {
        #print "Time = $year1/$mon1/$day1 at $hour1:$min1:$sec1\n";
        my $fulldate = $year1 . $mon1 . $day1;
        if ($fulldate >= $startdate && $fulldate <= $enddate) {
          if (($fulldate == $startdate && $hour1 < $starttime) ||
              ($fulldate == $enddate && $hour1 > $endtime) ) {
            #print "$lev is out of range\n";
            next;
          }
          #print "copying $lev to $thistargetdir\n";
          if (!-e $thistargetdir) {`mkdir -p $thistargetdir`;}
          `/bin/cp $fpath $thistargetdir`;
        }

      }
      
    }
  }
#code_index.lb.20021030_0601
#2002-11-12_at_17-29-12-UTC.xml.gz   
#20021111-172025.netcdf.gz 
}
